# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "databoxedge device get-extended-information",
)
class GetExtendedInformation(AAZCommand):
    """Gets additional information for the specified Azure Stack Edge/Data Box Gateway device.

    :example: Get extended information
        az databoxedge device get-extended information --device-name name -g rg
    """

    _aaz_info = {
        "version": "2023-07-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.databoxedge/databoxedgedevices/{}/getextendedinformation", "2023-07-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.device_name = AAZStrArg(
            options=["--device-name"],
            help="The device name.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.DevicesGetExtendedInformation(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class DevicesGetExtendedInformation(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataBoxEdge/dataBoxEdgeDevices/{deviceName}/getExtendedInformation",
                **self.url_parameters
            )

        @property
        def method(self):
            return "POST"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "deviceName", self.ctx.args.device_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2023-07-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _schema_on_200.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.properties
            properties.channel_integrity_key_name = AAZStrType(
                serialized_name="channelIntegrityKeyName",
            )
            properties.channel_integrity_key_version = AAZStrType(
                serialized_name="channelIntegrityKeyVersion",
            )
            properties.client_secret_store_id = AAZStrType(
                serialized_name="clientSecretStoreId",
            )
            properties.client_secret_store_url = AAZStrType(
                serialized_name="clientSecretStoreUrl",
            )
            properties.cloud_witness_container_name = AAZStrType(
                serialized_name="cloudWitnessContainerName",
                flags={"read_only": True},
            )
            properties.cloud_witness_storage_account_name = AAZStrType(
                serialized_name="cloudWitnessStorageAccountName",
                flags={"read_only": True},
            )
            properties.cloud_witness_storage_endpoint = AAZStrType(
                serialized_name="cloudWitnessStorageEndpoint",
                flags={"read_only": True},
            )
            properties.cluster_witness_type = AAZStrType(
                serialized_name="clusterWitnessType",
                flags={"read_only": True},
            )
            properties.device_secrets = AAZDictType(
                serialized_name="deviceSecrets",
                flags={"read_only": True},
            )
            properties.encryption_key = AAZStrType(
                serialized_name="encryptionKey",
            )
            properties.encryption_key_thumbprint = AAZStrType(
                serialized_name="encryptionKeyThumbprint",
            )
            properties.file_share_witness_location = AAZStrType(
                serialized_name="fileShareWitnessLocation",
                flags={"read_only": True},
            )
            properties.file_share_witness_username = AAZStrType(
                serialized_name="fileShareWitnessUsername",
                flags={"read_only": True},
            )
            properties.key_vault_sync_status = AAZStrType(
                serialized_name="keyVaultSyncStatus",
            )
            properties.resource_key = AAZStrType(
                serialized_name="resourceKey",
                flags={"read_only": True},
            )

            device_secrets = cls._schema_on_200.properties.device_secrets
            device_secrets.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.device_secrets.Element
            _element.encrypted_secret = AAZObjectType(
                serialized_name="encryptedSecret",
            )
            _element.key_vault_id = AAZStrType(
                serialized_name="keyVaultId",
            )

            encrypted_secret = cls._schema_on_200.properties.device_secrets.Element.encrypted_secret
            encrypted_secret.encryption_algorithm = AAZStrType(
                serialized_name="encryptionAlgorithm",
                flags={"required": True},
            )
            encrypted_secret.encryption_cert_thumbprint = AAZStrType(
                serialized_name="encryptionCertThumbprint",
                flags={"secret": True},
            )
            encrypted_secret.value = AAZStrType(
                flags={"required": True},
            )

            system_data = cls._schema_on_200.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            return cls._schema_on_200


class _GetExtendedInformationHelper:
    """Helper class for GetExtendedInformation"""


__all__ = ["GetExtendedInformation"]
