# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, Optional, TYPE_CHECKING

from azure.core.rest import HttpRequest, HttpResponse
from azure.mgmt.core import ARMPipelineClient
from msrest import Deserializer, Serializer

from . import models
from ._configuration import TrafficManagerManagementClientConfiguration
from .operations import EndpointsOperations, GeographicHierarchiesOperations, HeatMapOperations, ProfilesOperations, TrafficManagerUserMetricsKeysOperations

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials import TokenCredential

class TrafficManagerManagementClient:
    """TrafficManagerManagementClient.

    :ivar endpoints: EndpointsOperations operations
    :vartype endpoints: azure.mgmt.trafficmanager.operations.EndpointsOperations
    :ivar profiles: ProfilesOperations operations
    :vartype profiles: azure.mgmt.trafficmanager.operations.ProfilesOperations
    :ivar geographic_hierarchies: GeographicHierarchiesOperations operations
    :vartype geographic_hierarchies:
     azure.mgmt.trafficmanager.operations.GeographicHierarchiesOperations
    :ivar heat_map: HeatMapOperations operations
    :vartype heat_map: azure.mgmt.trafficmanager.operations.HeatMapOperations
    :ivar traffic_manager_user_metrics_keys: TrafficManagerUserMetricsKeysOperations operations
    :vartype traffic_manager_user_metrics_keys:
     azure.mgmt.trafficmanager.operations.TrafficManagerUserMetricsKeysOperations
    :param credential: Credential needed for the client to connect to Azure.
    :type credential: ~azure.core.credentials.TokenCredential
    :param subscription_id: Gets subscription credentials which uniquely identify Microsoft Azure
     subscription. The subscription ID forms part of the URI for every service call.
    :type subscription_id: str
    :param base_url: Service URL. Default value is 'https://management.azure.com'.
    :type base_url: str
    """

    def __init__(
        self,
        credential: "TokenCredential",
        subscription_id: str,
        base_url: str = "https://management.azure.com",
        **kwargs: Any
    ) -> None:
        self._config = TrafficManagerManagementClientConfiguration(credential=credential, subscription_id=subscription_id, **kwargs)
        self._client = ARMPipelineClient(base_url=base_url, config=self._config, **kwargs)

        client_models = {k: v for k, v in models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.endpoints = EndpointsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.profiles = ProfilesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.geographic_hierarchies = GeographicHierarchiesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.heat_map = HeatMapOperations(self._client, self._config, self._serialize, self._deserialize)
        self.traffic_manager_user_metrics_keys = TrafficManagerUserMetricsKeysOperations(self._client, self._config, self._serialize, self._deserialize)


    def _send_request(
        self,
        request,  # type: HttpRequest
        **kwargs: Any
    ) -> HttpResponse:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = client._send_request(request)
        <HttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/python/protocol/quickstart

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.HttpResponse
        """

        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, **kwargs)

    def close(self):
        # type: () -> None
        self._client.close()

    def __enter__(self):
        # type: () -> TrafficManagerManagementClient
        self._client.__enter__()
        return self

    def __exit__(self, *exc_details):
        # type: (Any) -> None
        self._client.__exit__(*exc_details)
